<?php
/** @noinspection PhpDeprecationInspection */
declare (strict_types = 1);

namespace EuKit\Base\ViewModels;

use EuKit\Base\Interfaces\ViewModel;
use Infomaker\Everyware\Base\Menus;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Support\Storage\Cache;

class Page implements ViewModel
{
    /**
     * Contains the article to fetch data from
     *
     * @var GenericPropertyObject
     */
    protected $viewData;

    /**
     * ViewModel constructor.
     *
     * @param GenericPropertyObject $object
     */
    public function __construct(GenericPropertyObject $object)
    {
        $this->viewData = $object;
        $this->viewData->fill([
            'topmenu'  => $this->getMenuItems('top-menu'),
            'menu'     => $this->getMenuItems('main-menu'),
            'logo'     => $this->getLogo('custom_logo'),
            'logo_nav' => $this->getLogo('custom_logo_nav'),
        ]);
    }

    /**
     * Retrieve data to be used in the View
     *
     * @return array
     */
    public function getViewData(): array
    {
        return $this->viewData->toArray();
    }

    public function setViewData($property, $data): void
    {
        $this->viewData->fill([$property => $data]);
    }

    protected function getMenuItems(string $menuName)
    {
        $menu = Menus::getFromLocation($menuName);
        return $menu ? $menu->items(true) : [];
    }

    /**
     * @param string $logoName  An option
     *
     * @return array  ['src'=>string, 'width'=>int, 'height'=>int] or []
     */
    protected function getLogo(string $logoName): array
    {
        $cacheKey = $logoName . '_post_id';
        $postId = Cache::get($cacheKey);
        if ($postId === false) {

            $url = get_option($logoName);
            if ($url === false) {
                return [];
            }

            $postId = $this->getAttachmentPostIdByUrl($url);
            $ttl = 60;
            Cache::set($cacheKey, $postId, $ttl);
        }

        if (empty($postId)) {
            return [];
        }

        $meta = wp_get_attachment_metadata($postId);
        $src = wp_get_attachment_url($postId);
        return [
            'src'    => $src,
            'width'  => $meta['width'],
            'height' => $meta['height'],
            'srcset' => wp_calculate_image_srcset([$meta['width'], $meta['height']], $src, $meta, $postId),
            'sizes'  => wp_calculate_image_sizes([$meta['width'], $meta['height']], $src, $meta, $postId),
        ];
    }

    private function getAttachmentPostIdByUrl(string $url): ?int
    {
        // If plugin deliciousbrains/wp-amazon-s3-and-cloudfront is active, $url has probably been filtered and needs to be reverted.
        $url = apply_filters('as3cf_filter_post_provider_to_local', $url);

        // Extract 'year/month/filename.suffix' from 'https://domain/blablabla/year/month/filename.suffix'.
        $attachmentFilePath = implode('/', array_slice(explode('/', $url), -3));

        $posts = get_posts([
            'post_type'  => 'attachment',
            'meta_query' => [
                [
                    'key'   => '_wp_attached_file',
                    'value' => $attachmentFilePath
                ]
            ]
        ]);

        if (count($posts) > 1) {
            $message = sprintf('Found more than one attachment matching the URL `%s`', $url);
            NewRelicLog::logException(new \UnexpectedValueException($message));
        }

        return (count($posts) > 0) ? $posts[0]->ID : null;
    }
}
